<?php

/* ************************************************* *\
|* ================================================= *|
|* =       Copyright (c) 2012 Jacob Pollack.       = *|
|* ================================================= *|
|* = THIS SOFTWARE IS PRIVATE AND CANNOT BE FOUND  = *|
|* = ON THE PUBLIC DOMAIN. ANY DISTRIBUTION OR USE = *|
|* = OF THIS SOFTWARE WITHOUT THE EXPLICIT         = *|
|* = PERMISSION OF THE AUTHOR WILL RESULT IN LEGAL = *|
|* = ACTION BEING TAKEN.                           = *|
|* =                                               = *|
|* = THIS SOFTWARE IS CREATED FOR EDUCATIONAL      = *|
|* = PURPOSES ONLY. CHANGES OF ANY SORT MUST BE    = *|
|* = SUBMITTED TO THE RESPOSITORY ACCORDINGLY.     = *|
|* ================================================= *|
|* = @author    Jacob Pollack                      = *|
|* = @email     jacobpollack@jamach.com            = *|
|* = @date      05/08/2012 (DD/MM/YYYY)            = *|
|* =                                               = *|
|* = @contributors  none                           = *|
|* ================================================= *|
\*************************************************** */

/**
 * The module class which handles the loading and invoking
 * of all of the modules found in the 'module' folder.
 * 
 * @name    Module Handler
 * @author  Jacob Pollack
 */

final class modulehandler
{
    /**
     * Path to the modules directory.
     * 
     * @var string
     */
    private $path = 'includes/modules/';
    
    /**
     * Name of the abstract class each module will extend from.
     * 
     * @var string
     */
    private $skeleton = 'module';
    
    /**
     * 'List' of the module class names.
     * 
     * @var array
     */
    private $modules = array();
    
    /**
     * 'List' of the module objects.
     * 
     * @var array
     */
    private $module_objects = array();
    
    /**
     * Class constructor, called upon instantiation.
     * 
     * @param   boolean     $validate   Whether or not to load any modules.
     * 
     * @return  void
     */
    final public function __construct($validate)
    {
        if ($validate == true)
        {
            $this->path = CWD . $this->path;
            
            $this->load_modules();
        }
        else
        {
            // Do not load modules, log this action.
        }
    }
    
    /**
     * Loads all of the modules in the 'modules' folder. It then requests to
     * invoke the module. Supports namespaces. Will grab the modules list from
     * the cache, if it exists.
     * 
     * @return  void
     */
    final public function load_modules()
    {
        global $quasar;
        
        // Checks if cache exists for 'modules' array.
        if ($quasar->object->cachehandler->file_exists('modules') == false)
        {
            $dir = scandir($this->path, 1);

            // The '2' is found from the '.' and '..'.
            if (sizeof($dir) > 2)
            {
                pop($dir, 2);

                // Loops through the directory and finds all of the modules then
                // loads them into an array.
                foreach ($dir as $file)
                {
                    require_once $this->path . $file;

                    // Removes the '.php' extension.
                    $name = substr($file, 0, -4);
                    // Class name.
                    $class = null;
                    // Splits namespaces.
                    $class_parts = explode('/', $name);

                    // Grabs the class name from the namespace, if exists.
                    if (sizeof($class_parts) > 1)
                    {
                        $class = end($class_parts);
                    }
                    else
                    {
                        $class = current($class_parts);
                    }

                    // Puts the class name in the modules array.
                    $this->modules[] = $class;
                }
                
                // Stores in the cache.
                $quasar->object->cachehandler->file_store('modules', $this->modules);
            }
            else
            {
                // No modules to load, log this action.
            }
        }
        else
        {
            // Fetches the modules array from the cache.
            $this->modules = $quasar->object->cachehandler->file_fetch('modules');
        }
        
        // Checks if there are any modules to load.
        if (sizeof($this->modules) > 0)
        {
            // Loops through the modules and includes them.
            foreach ($this->modules as $class)
            {
                require_once $this->path . $class . '.php';

                $object = new $class();

                // Checks if the object extends the proper skeleton.
                if (is_subclass_of($object, $this->skeleton) == true)
                {
                    $this->module_objects[] = $object;
                }
                else
                {
                    // Log this error and spit out a warning for the module.
                }

                // Deletes the module object.
                delete($object);
            }
                
            // Sorts the array of modules by order number.
            usort($this->module_objects, array('modulehandler', 'cmp'));

            // Loops through the array of loaded modules objects and invokes them.
            //foreach ($this->module_objects as $module)
            //{
            //    $this->invoke($module);
            //}
            array_map(array('modulehandler', 'invoke'), $this->module_objects);
        }
    }
    
    /**
     * Invokes a module. Validates that it is a part of the 'module' abstract class
     * then loads it.
     *  
     * @param   object  $object Object passed by reference.
     * 
     * @return  void
     */
    final private function invoke(module &$object)
    {           
        if ($object->validate() == true)
        {
            $object->load();
        }
        else
        {
            // Did not load the module because it did not validate properly.
        }
    }
    
    /**
     * Method callback used from 'usort' to sort the array of objects by their order
     * number.
     * 
     * @param   object  $a  The first comparable object.
     * @param   object  $b  The second comparable object.
     * 
     * @return  integer
     */
    final private function cmp(module $a, module $b)
    {   
        return strcmp($a->get_order(), $b->get_order());
    }
    
    /**
     * Empty destructor.
     * 
     * @return  void
     */
    final public function __destruct() { }
}

?>